import re
from wsgiref.headers import Headers

STATICS_FILE_MIME_TYPE = (
    (("png", "jpg", "jpeg", "gif"), "image", None),
    (("js", "woff"), "application", ("javascript", "x-font-woff")),
    (("css", ), "text", None),
    (("mp3", "wav", ), "audio/mpeg", None),
)


class BaseResponse(object):
    def __init__(self, *args, **kwargs):
        self.status_code = kwargs.get("status_code", 200)
        self.reason_phrase = kwargs.get("reason_phrase", "OK")
        self.cookies = []
        self.charset = kwargs.get("charset", "utf-8")
        self._handler_class = None
        self._content = []
        self.__set_default_headers()
        self.headers.add_header(
            "Content-Type",
            kwargs.get("content_type", "text/html; charset=%s" % self.charset)
        )

    def __set_default_headers(self):
        self.headers = Headers([
            ("Server", "MadLiar"),
            ("Access-Control-Allow-Origin", "*"),
            ("X-Frame-Options", "SAMEORIGIN"),
        ])

    @property
    def content(self):
        return b''.join(self._content)

    @content.setter
    def content(self, value):
        # Consume iterators upon assignment to allow repeated iteration.
        if not hasattr(value, '__iter__'):
            value = [value]

        content = b''.join(map(
            lambda x: bytes(x) if isinstance(x, bytes) else bytes(x.encode(self.charset)),
            value
        ))
        self.headers.add_header("Content-length", str(len(content)))
        self._content = [content]

    def __iter__(self):
        return iter(self._content)


class HttpResponse(BaseResponse):
    def __init__(self, *args, **kwargs):
        BaseResponse.__init__(self, *args, **kwargs)
        content = kwargs.get("content", args[0])
        if type(content) == unicode:
            content = content.encode("utf-8")
        if type(content) in (bytes, str):
            self.content = content


class Http404Response(BaseResponse):
    def __init__(self, *args, **kwargs):
        BaseResponse.__init__(self, *args, **kwargs)
        self.content = "<center><h3>404 Not Found!</h3></center>"
        self.status_code = 404
        self.reason_phrase = "Not Found"


class Http410Response(BaseResponse):
    def __init__(self, *args, **kwargs):
        BaseResponse.__init__(self, *args, **kwargs)
        self.content = "<center><h3>410 Gone!</h3></center>"
        self.status_code = 410
        self.reason_phrase = "Gone"


class StreamingHttpResponse(BaseResponse):
    def _set_streaming_content(self, value):
        return self, value


class FileResponse(StreamingHttpResponse):
    """
    A streaming HTTP response class optimized for files.
    """
    block_size = 4096

    def _set_streaming_content(self, value):
        super(FileResponse, self)._set_streaming_content(value)


class HttpResponseRedirectBase(HttpResponse):
    allowed_schemes = ['http', 'https', 'ftp']

    def __init__(self, redirect_to, *args, **kwargs):
        super(HttpResponse, self).__init__()
        self._content_type_for_repr = ""
        self._redirect_to = redirect_to

    url = property(lambda self: self['Location'])

    def __repr__(self):
        return '<%(cls)s status_code=%(status_code)d%(content_type)s, url="%(url)s">' % {
            'cls': self.__class__.__name__,
            'status_code': self.status_code,
            'content_type': self._content_type_for_repr,
            'url': self.url,
        }


def static_files_response(request, static_path):
    static_file_path = static_path + request.route_path
    try:
        with open(static_file_path, "rb") as f:
            content = f.read()
    except IOError:
        return Http404Response()

    try:
        static_file_ext_name = re.match(r".*\.(.*)", static_file_path).groups()[0].lower()
    except AttributeError:
        return Http404Response()

    for static_file_typedef in STATICS_FILE_MIME_TYPE:
        ext_name_group, mime_type, mime_desc = static_file_typedef
        if static_file_ext_name in ext_name_group:
            content_type = "%s/%s" % (
                mime_type,
                mime_desc[ext_name_group.index(static_file_ext_name)] if mime_desc else static_file_ext_name
            )
            break
    else:
        content_type = None

    return HttpResponse(content, content_type=content_type)



import os
import sys


class ManagementUtility(object):
    """
    Encapsulates the logic of the command tools utilities.

    A ManagementUtility finds out the command in the lib.command.py file
    and run it.
    """
    def __init__(self, argv=None):
        self.argv = argv or sys.argv[:]
        self.prog_name = os.path.basename(self.argv[0])
        self.settings_exception = None
        print "self.argv: %s" % self.argv

    def main_help_text(self):
        """
        Returns the script's main help text, as a string.
        """
        return ""

    def execute(self):
        """
        Given the command-line arguments, this figures out which subcommand is
        being run, and runs it.
        """
        try:
            subcommand = self.argv[1]
        except IndexError:
            subcommand = 'runserver'  # Display help if no arguments were given.

        if subcommand == "runserver":
            try:
                host, port = self.argv[2].split(":")
            except (IndexError, TypeError) as e:
                error_msg = "An error happend: %s, use default configured.\n" % str(e)
                sys.stdout.write(error_msg)
                host, port = "0.0.0.0", 8080

            sys.stdout.write("run as simple wsgi server on %s:%s.\n" % (host, port))
            from madliar.core import wsgi_server
            wsgi_server(host, int(port)).serve_forever()

        else:
            from lib import command
            try:
                return getattr(command, subcommand)(*self.argv[2:])
            except AttributeError:
                sys.stderr.write("Unknown command: '%s' \nType 'help' for usage." % subcommand)


def execute_from_command_line(argv=None):
    """
    A simple method that runs a Manager.
    """
    utility = ManagementUtility(argv)
    utility.execute()



import re

from importlib import import_module
from wsgiref.simple_server import make_server
from madliar.response import static_files_response, Http404Response
from madliar.request import WSGIRequest
from etc.config import DEBUG, STATICS_URL_MAP, CUSTOM_MIDDLEWARE
from application.urls import url as user_url_map


def dynamic_import_class(dotted_path):
    """
    Import a dotted module path and return the attribute/class designated by the
    last name in the path. Raise ImportError if the import failed.
    """
    try:
        module_path, class_name = dotted_path.rsplit('.', 1)
    except ValueError:
        raise ImportError("%s doesn't look like a module path" % dotted_path)

    dynamic_imported_module = import_module(module_path)

    try:
        return getattr(dynamic_imported_module, class_name)
    except AttributeError:
        raise ImportError(
            """Module "%s" does not define a "%s" attribute/class"""
            % (module_path, class_name)
        )


class BaseHandler(object):
    def __init__(self, *args, **kwargs):
        pass


class WSGIHandler(BaseHandler):
    request_class = WSGIRequest
    __middleware_chain = None

    def __init__(self, *args, **kwargs):
        super(WSGIHandler, self).__init__(*args, **kwargs)
        self._handler = None

    def __call__(self, environ, start_response):
        request = self.request_class(environ)
        response = self.get_response(request)

        response._handler_class = self.__class__

        status = '%d %s' % (response.status_code, response.reason_phrase)
        start_response(str(status), response.headers.items())

        if getattr(response, 'file_to_stream', None) is not None and environ.get('wsgi.file_wrapper'):
            response = environ['wsgi.file_wrapper'](response.file_to_stream)
        return response

    def _load_middleware(self):
        if self.__class__.__middleware_chain is None:
            get_response_func = self.route_distributing
            for class_path in CUSTOM_MIDDLEWARE[::-1]:
                cls = dynamic_import_class(class_path)
                if callable(cls):
                    get_response_func = cls(get_response_func).__call__
            self.__class__.__middleware_chain = get_response_func

    def route_distributing(self, request):
        search_loop = user_url_map.items()
        route_path = request.route_path
        while True:
            try:
                url, sub_url_map = search_loop.pop(0)
            except IndexError:
                break

            m = re.match(url, route_path)
            if m:
                if callable(sub_url_map):
                    return sub_url_map(request, *m.groups())

                route_path = route_path[len(m.group()):] or "/"
                search_loop = sub_url_map.items()

        # Not hit
        if DEBUG:
            for url, static_path in STATICS_URL_MAP.items():
                m = re.match(url, request.path_info)
                if m:
                    request.route_path = request.path_info[len(m.group()):] or "/"
                    return static_files_response(request, static_path)
        return Http404Response()

    def get_response(self, request):
        self._load_middleware()
        return self.__class__.__middleware_chain(request)


def get_application():
    return WSGIHandler()


def wsgi_server(host, port):
    """Start a simple server."""
    return make_server(host, port, get_application())



"""
Base middleware

"""


class BaseMiddleware(object):
    def __init__(self, get_response):
        # One-time configuration and initialization.
        self.get_response = get_response

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.

        response = self.get_response(request)

        # Code to be executed for each request/response after
        # the view is called.

        return response



from setuptools import setup, find_packages  
  
setup(  
    name='madliar',
    version='0.1a1.dev1023',
    description='A tiny WSGI freamwork.',
    classifiers=[  
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',  
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
    # install_requires=['jinja2'],
    entry_points={
        'console_scripts': [
            'madliar-manage.py=madliar.management:execute_from_command_line',
        ],
    },
    author='caoliang',
    url='https://github.com/cl-ei',
    author_email='i@caoliang.net',
    license='MIT',  
    packages=find_packages(),
    include_package_data=False,  
    zip_safe=True,  
)

